function FluidListController($scope, $element, ApplicationContext, restEngine) {

	/*
	 * -> fluidList
	 * The widget with this template in the container json
	 *
	 * -> fluidListDOM
	 * The DOM element representing the fluid list template
	 *
	 * -> actionEvents
	 * Array containing the code element from all events with name 'ActionEvent' defined on the jobson.json widget declaration
	 */
	var self = this;
	var fluidListDOM, fluidList;
	this.actionEvents = [];

	/*
	 * Initializes the variables, check if this widget should be visible and calls the createFLuidListElements method.
	 *
	 * returns void
	 */
	this.initializeFluidList = function() {
		fluidList = $scope.widget;
		fluidListDOM = $element.get()[0];

		if (!fluidList.isVisible)
			fluidListDOM.style.visibility = "hidden";

		fluidListDOM.style.columnCount = Math.floor(Math.sqrt(fluidListDOM.clientWidth)/7);

		this.actionEvents = fluidList.events.filter(function(event) {
			return event.name == "ActionEvent";
		});

		pullData().then(createFluidListElements.bind(null, fluidList, fluidListDOM));
	};

	/*
	 * Populates the fluidList.dataSource.data array with data from the backend.
	 * The backend MUST return an object named 'dataset' with an array property named 'data'
	 *
	 * This method orders the data as first element in 0 position, second element in half position, third in 1 position, fourth in half+1 position...
	 * This is due the div columns functionality. Data must be ordered this way for being displayed the first and second element side by side, third and fourth
	 * side by side and so on.
	 *
	 * returns void
	 */
	 var pullData = function() {
		if (!fluidList.dataSource)
			fluidList.dataSource = {};
		if (!fluidList.dataSourceFilter)
			fluidList.dataSourceFilter = [];

		return restEngine.post(fluidList.dataSource.name, {
			"requestType": "FilterData",
			"filter": fluidList.dataSourceFilter
		}).then(function(fluidList, response) {
			var length = response.dataset.data.length;
			for (var i=0, j=0; i<length/2; i++, j+=2) {
				fluidList.dataSource.data[i] = response.dataset.data[j];
				fluidList.dataSource.data[length/2+i] = response.dataset.data[j+1];
			}
			if (length%2 !== 0)
				fluidList.dataSource.data.push(response.dataset.data[length-1]);
			
			// fluidList.dataSource.data = response.dataset.data;
		}.bind(null, fluidList), function(error) {
			console.log("%cFluidList could not retrieve data from given datasource", "background-color: red; color: black;");
		});
	 };

	/*
	 * Iterates over the fluidList.fields array and create a DOM element for each one of them.
	 *
	 * returns void
	 */
	var createFluidListElements = function(/*object*/ fluidList, /*DOM object*/ fluidListDOM) {
		var fluidListElement, fluidListElementTemplate;

		document.querySelector(".container").style.overflowY = "scroll";
		fluidListElementTemplate = fluidListDOM.firstElementChild;

		for (var i=0; i<fluidList.dataSource.data.length; i++) {
			fluidListElement = fluidListElementTemplate.cloneNode(true);

			fluidListElement.classList.add("rui-fluid-list-element");
			fluidListElement.setAttribute("id", fluidList.dataSource.data[i].name);
			fluidListElement.style.height = (fluidList.dataSource.data[i].height * 15 + 100) + "px";
			fluidListElement.lastElementChild.innerHTML = fluidList.dataSource.data[i].label;
			fluidListElement.style.backgroundImage = "url('"+fluidList.dataSource.data[i].image+"')";
			fluidListElement.addEventListener("click", executeActionEvent.bind(self));

			fluidListDOM.appendChild(fluidListElement);
		}

		fluidListDOM.removeChild(fluidListElementTemplate);
	};

	/*
	 * Evaluates the strings in actionEvents.
	 *
	 * returns void
	 */
	var executeActionEvent = function(/*object*/ event) {
		for (var i=0; i<this.actionEvents.length; i++)
			$scope.$eval(
				this.actionEvents[i].code,
				getApplicationContextWithEventArgs(event.currentTarget.getAttribute("id"))
			);
	};

	/*
	 * Adds to ApplicationContext an element called "args" with reference to the event owner.
	 *
	 * returns object
	 */
	var getApplicationContextWithEventArgs = function(/*string*/ eventTargetName) {
		ApplicationContext.args = {
			owner: filterElementsByName(fluidList.dataSource.data, eventTargetName)
		};

		return ApplicationContext;
	};

	/*
	 * Filters the fluidList.dataSource.data array in search of an object with the name passed as parameter.
	 * Returns the first one found.
	 *
	 * returns object
	 */
	var filterElementsByName = function(/*object array*/ array, /*string*/ name) {
		return array.filter(function(element) {
			return element.name == name;
		})[0];
	};
}