function CalendarCtrl($scope) {
 
	var widget = $scope.widget;
	$scope.eventSources = [];

	var convertToDate = function(strDate) {
		var result;
		if (strDate) {
			strDate = strDate.split("/").reverse().join("-");
			var parts = strDate.match(/(\d+)/g);
			result = new Date(parts[0], parts[1] - 1, parts[2]);
		}
		return result;
	};

	var prepareEvents = function(data) {
		var titleKey = widget.titleEvents,
			startKey = widget.startEvents,
			endKey = widget.endEvents;

		angular.forEach(data, function(value, key) {

			data[key].title = data[key][titleKey];
			data[key].start = convertToDate(data[key][startKey]);

			if (data[key][endKey]) {
				data[key].end = convertToDate(data[key][endKey]);
			}

		});
		return data;
	};

	$scope.$watch('widget.dataSource.data', function() {
		if (widget.dataSource.data !== undefined) {
			$scope.events = prepareEvents(widget.dataSource.data);
			$scope.eventSources.push($scope.events);
		}
	});

	/* event source that contains custom events on the scope */

	/* event source that calls a function on every view switch */
	$scope.eventsF = function(start, end, callback) {
		var m = new Date(start).getMonth();
		var e = new Date(end).getTime() / 1000;
		var s = new Date(start).getTime() / 1000;
		var events = [{
			title: 'Feed Me ' + m,
			start: s + (50000),
			end: s + (100000),
			allDay: false,
			className: ['customFeed']
    }];
		callback(events);
	};

	$scope.calEventsExt = {
		color: '#f00',
		textColor: 'yellow',
		events: [
//      {type:'party',title: 'Lunch',start: new Date(y, m, d, 12, 0),end: new Date(y, m, d, 14, 0),allDay: false},
//      {type:'party',title: 'Lunch 2',start: new Date(y, m, d, 12, 0),end: new Date(y, m, d, 14, 0),allDay: false},
//      {type:'party',title: 'Click for Google',start: new Date(y, m, 28),end: new Date(y, m, 29),url: 'http://google.com/'}
    ]
	};

	/* alert on eventClick */
	$scope.alertOnEventClick = function(event, allDay, jsEvent, view) {
		$scope.alertMessage = (event.title + ' was clicked ');
	};
	/* alert on Drop */
	$scope.alertOnDrop = function(event, dayDelta, minuteDelta, allDay, revertFunc, jsEvent, ui, view) {
		$scope.alertMessage = ('Event Droped to make dayDelta ' + dayDelta);
	};
	/* alert on Resize */
	$scope.alertOnResize = function(event, dayDelta, minuteDelta, revertFunc, jsEvent, ui, view) {
		$scope.alertMessage = ('Event Resized to make dayDelta ' + minuteDelta);
	};
	/* add and removes an event source of choice */
	$scope.addRemoveEventSource = function(sources, source) {
		var canAdd = 0;
		angular.forEach(sources, function(value, key) {
			if (sources[key] === source) {
				sources.splice(key, 1);
				canAdd = 1;
			}
		});
		if (canAdd === 0) {
			sources.push(source);
		}
	};
	/* add custom event*/
	$scope.addEvent = function() {
		$scope.events.push({
			title: 'Open Sesame',
			start: new Date(y, m, 28),
			end: new Date(y, m, 29),
			className: ['openSesame']
		});
	};

	/* remove event */
	$scope.remove = function(index) {
		$scope.events.splice(index, 1);
	};

	$scope.setTitle = function(calendar) {
		$scope.titleCalendar = calendar.fullCalendar('getView').title;
	};

	/* Change View */
	$scope.changeView = function(view, calendar) {
		calendar.fullCalendar('changeView', view);
		$scope.titleCalendar = calendar.fullCalendar('getView').title;
	};

	$scope.next = function(calendar) {
		calendar.fullCalendar('next');
		$scope.titleCalendar = calendar.fullCalendar('getView').title;
	};
	$scope.prev = function(calendar) {
		calendar.fullCalendar('prev');
		$scope.titleCalendar = calendar.fullCalendar('getView').title;
	};
	$scope.goToday = function(calendar) {
		calendar.fullCalendar('today');
		$scope.titleCalendar = calendar.fullCalendar('getView').title;
	};

	/* Change View */
	$scope.renderCalendar = function(calendar) {
		if (calendar) {
			calendar.fullCalendar('render');
		}
	};

	/* config object */
	$scope.uiConfig = {
		calendar: {
			height: 450,
			editable: true,
			header: {
				left: '',
				center: 'title',
				right: 'today prev,next'
			},
			lang: "pt",
			monthNames: ['Janeiro', 'Fevereiro', 'Março', 'Abril', 'Maio', 'Junho', 'Julho', 'Agosto', 'Setembro', 'Outubro', 'Novembro', 'Dezembro'],
			monthNamesShort: ['Jan', 'Fev', 'Mar', 'Abr', 'Mai', 'Jun', 'Jul', 'Ago', 'Set', 'Out', 'Nov', 'Dez'],
			dayNames: ["Domingo", "Segunda", "Terça", "Quarta", "Quinta", "Sexta", "Sabado"],
			dayNamesShort: ["Dom", "Seg", "Ter", "Qua", "Qui", "Sex", "Sab"],
			eventClick: $scope.alertOnEventClick,
			eventDrop: $scope.alertOnDrop,
			eventResize: $scope.alertOnResize,
			columnFormat: {
				month: 'ddd',
				week: 'ddd dd/MM',
				day: 'dddd dd/MM/yyyy'
			},
			titleFormat: {
				month: 'MMMM yyyy',
				week: "d [MMMM][ yyyy]{ '-' d MMMM yyyy}",
				day: 'dddd dd/MM/yyyy'
			},
			timeFormat: {
				agenda: 'H:mm{ - h:mm}'
			}
		}
	};

	$scope.initMonthTitle = function() {
		var monthIndex = new Date().getMonth();
		var year = new Date().getFullYear();
		$scope.titleCalendar = $scope.uiConfig.calendar.monthNames[monthIndex] + " " + year;
	};

}