var RequestDataSourceEngine = function(requestEngine, requestFactory, MethodBuilder, ScreenService, ZHPromise, ZhUtil, dataSourceHolder, templateManager) {
	var returnItem = function(item) {
		return item.map(function(item) {
			return '"' + item + '"';
		}).join(',');
	};
	var buildMethods = function(methods) {
		for (var i in methods) {
			if (methods.hasOwnProperty(i)) {
				var method = methods[i];
				var params = returnItem(method.parameters);
				var code = method.name + "(" + params + ");";
				MethodBuilder.build(code, {
					event: {
						code: code
					}
				})();
			}
		}
	};
	var showMessages = function(messages) {
		if (messages) {
			ScreenService.showMessage(messages[0].message, messages[0].type, undefined, messages[0].variables);
		}
	};
	var showNotifications = function(notifications) {
		if (notifications) {
			notifications.forEach(function(notification) {
				ScreenService.notificationMessage(notification.message, notification.type, undefined, notification.variables);
			});
		}
	};
	var _getDataSourceProcessError = function(errorCallBack) {
		return function(e) {
			if (errorCallBack) {
				errorCallBack(e);
			}
		};
	};
	var _getDataSourceSetResult = function(dataSourceName, callBack) {
		return function(data) {
			if (data.dataset && Util.isFunction(callBack)) {
				var keys = Object.keys(data.dataset);
				if (keys.length === 1) {
					var datasetNameFormated = keys[0];
					if (datasetNameFormated.indexOf('/') !== 0) {
						datasetNameFormated = '/' + datasetNameFormated;
					}

					callBack(data.dataset[keys[0]], datasetNameFormated);
				} else {
					callBack(data.dataset, keys);
				}
			}

			showMessages(data.messages);
			showNotifications(data.notifications);
			buildMethods(data.method);
		};
	};

	function isOrderBy(filter) {
		return Util.isObject(filter);
	}

	function removePrefixOnOrderBy(filter) {
		filter.name = removePrefixFromName(filter.name);
		return filter;
	}

	function removePrefixOrderAndGroupBy(filters) {
		return filters.map(function(filter) {
			if (isOrderBy(filter)) {
				filter = removePrefixOnOrderBy(filter);
			} else {
				filter = removePrefixFromName(filter);
			}

			return filter;
		});
	}

	function removePrefixFromName(name) {
		if (name.indexOf('__proxy_') === 0) {
			name = name.substr(8);
		}

		return name;
	}

	function isOrderOrGroupBy(filter) {
		return Util.isArray(filter);
	}

	function removePrefix(name) {
		if (isOrderOrGroupBy(name)) {
			return removePrefixOrderAndGroupBy(name);
		} else {
			return removePrefixFromName(name);
		}
	}

	this.getDataSource = function(name, callBack, filter, page, itemsPerPage, errorCallBack, retryCount, timeout, retryCallback) {

		if(!Util.isFunction(callBack)){
			retryCallback = timeout;
			timeout = retryCount;
			retryCount = errorCallBack;
			errorCallBack = itemsPerPage;
			itemsPerPage = page;
			page = filter;
			filter = callBack;
		}

		var localFilter;
		if (!retryCount) {
			returnCount = 0;
		}
		if (!timeout) {
			timeout = templateManager.project.timeout;
		}
		if (filter === undefined) {
			filter = [];
		} else {
			localFilter = Util.clone(filter);
			localFilter.map(function(f) {
				f.name = removePrefix(f.name);
				f.value = ZhUtil.evaluate(f.value);
			});
		}
		if (page === undefined) {
			page = 1;
		}
		var processError = _getDataSourceProcessError(errorCallBack);
		var setResult = _getDataSourceSetResult(name, callBack);
		var request = requestFactory.factory({
			"requestType": "filterData",
			"filter": localFilter,
			"page": page,
			"itemsPerPage": itemsPerPage,
			"serviceName": name,
			"callBack": setResult,
			"errorCallBack": processError,
			"timeout": timeout,
			"retryCount": retryCount,
			"retryCallback": function() {
				if (retryCallback) {
					retryCallback();
				}
			}
		}, name, setResult);
		dataSourceHolder.getDataSource(name);
		return requestEngine.doRequest(request);
	};
	var _setDataSourceBuildCallback = function() {
		return function(data) {
			showMessages(data.messages);
			showNotifications(data.notifications);
			buildMethods(data.method);
		};
	};
	this.setDataSource = function(name, dataSource, callBack, retryCount, timeout, retryCallback) {
		var dataSourceRequest = dataSource.filter(function(item) {
			return item.changed;
		}).map(ZhUtil.clearRow);
		if (dataSourceRequest.length > 0) {
			var setResult = callBack ? callBack : _setDataSourceBuildCallback();
			var request = requestFactory.factory({
				"requestType": "dataSource",
				'dataSource': dataSourceRequest,
				"serviceName": name + "/save",
				"callBack": setResult,
				"timeout": timeout,
				"retryCount": retryCount,
				"retryCallback": function() {
					if (retryCallback) {
						retryCallback();
					}
				}
			}, name, setResult);
			return requestEngine.doRequest(request);
		} else {
			var deferred = ZHPromise.defer();
			deferred.resolve(dataSourceRequest);
			return deferred;
		}
	};
	this.save = this.setDataSource;
	var _removeSetResult = function(callBack) {
		return function(data) {
			if (callBack) {
				callBack(data);
			}
			showMessages(data.messages);
			showNotifications(data.notifications);
			buildMethods(data.method);
		};
	};
	this.remove = function(name, row, callBack) {
		var dataSource;
		if (Array.isArray(row)) {
			dataSource = row.map(ZhUtil.clearRow);
		} else {
			row = ZhUtil.clearRow(row);
		}
		var setResult = _removeSetResult(callBack);
		var request;
		if (dataSource) {
			request = requestFactory.factory({
				"requestType": "dataSource",
				'dataSource': dataSource,
				"serviceName": name + "/remove",
				"callBack": setResult
			}, name, setResult);
		} else {
			request = requestFactory.factory({
				"requestType": "Row",
				'row': row,
				"serviceName": name + "/remove",
				"callBack": setResult
			}, name, setResult);
		}
		return requestEngine.doRequest(request);
	};
	this.sync = function(name, callBack, filters){
		if(Util.isFunction(callBack)) {
			callBack([]);
		}
		var deferred = ZHPromise.defer();
		deferred.resolve([]);
		return deferred.promise;
	};
};