(function () {
	var ZeedhiDirectives = angular.module("ZeedhiDirectives");

	ZeedhiDirectives.directive(
		'zhFloatingControl',
		["templateManager", "ScreenService", "$timeout", "eventEngine", "ApplicationContext", "$window", "$rootScope", "ZhUtil", "$compile", "ZhKeyboardService", FilterFloatingControlDirective]
	);

	function FilterFloatingControlDirective(templateManager, ScreenService, $timeout, eventEngine, ApplicationContext, $window, $rootScope, zhUtil, $compile, ZhKeyboardService) {
		return {
			priority: 10,
			link: function($scope, element, attrs, controller) {
				var widget = $scope.widget;
				if (widget.floatingControl === false) return;

				var defaultOptions = {
					searchAction: Util.isUndefined(element.parent().attr('data-hide-search')) && Util.isUndefined(element.parent().attr('hide-search')),
					openSearch: false
				};
				widget.floatingControl = Util.extend(defaultOptions, widget.floatingControl);

				widget.floatingControl.addStatus({
					priority: 20,
					classString: 'searching',
					icon: 'search',
					conditionFunction: function() {
						return (widget.filterApplied && widget.filterApplied.length > 0) || Util.objectHasAnyValue($scope.searchCriteria);
					},
					infoTemplate: function() {
						var template;
						if (widget.floatingControl.customizationAction && widget.floatingControl.customizationAction.filter) {
							template = '<div class="searching-control-info"><span data-zh-icon="filter no-border icon-white" data-zh-touchstart="ControlInfoIconClick(\'filter\')"></span><div class="searching-control-info-details" data-ng-if="widget._filterInfo"><span data-ng-bind-html="widget._filterInfo"></span></div></div><div class="searching-control-info-text" data-ng-if="widget._filterInfo" data-zh-bind-translate="\'Filter applied\'"></div>';
						} else {
							template = '<div class="searching-control-info"><span data-zh-icon="close-x no-border icon-white" data-zh-touchstart="ControlInfoIconClick(\'clear\')"></span><div class="searching-control-info-details" data-ng-if="widget._filterInfo"><span data-ng-bind-html="widget._filterInfo"></span></div></div><div class="searching-control-info-text" data-ng-if="widget._filterInfo" data-zh-bind-translate="\'Filter applied\'"></div>';
						}
						return template;
					},
					scope: $scope
				});

				var isTheOnlyActionAvailable = function(propertyName){
					var actionsToCheck= ["configAction", "refreshAction", "xlsAction", "pdfAction", "actions", "searchAction"];
					var filtered = actionsToCheck.filter(function(property){
						var action = widget.floatingControl[property];
						return Util.isArray(action) ? action.length : action;
					});
					return (filtered.length == 1 && filtered[0] == propertyName);
				};

				$scope.$on('floating-control-before-close', function() {
					element.find('.search-action input').blur();
				});

				$scope.$on('floating-control-after-close', function() {
					if (element.hasClass('searching') || isTheOnlyActionAvailable("searchAction")) {
						element.find('.search-action').addClass('opened');
						Util.selectElement(element.find('.search-action input'));
					}
				});

				$scope.openSearchAction = function() {
					$timeout.cancel($scope._closeTimeout);
					if (!element.hasClass('opened')) {
						$scope.toggleActionMenu();
					}
					$scope.closeOpenedActions();
					var searchAction = element.find('.search-action').closest('.float-action');
					if (searchAction && searchAction.length > 0) {
						searchAction.find("span.search-action-icon").click();
						Util.selectElement(searchAction.find('input'));
					}
				};

				if (widget.floatingControl.openSearch) {
					$timeout(function() {
						var floatingControl = (ScreenService.isPopupOpen() ? $('.popup-container .container:visible') : $('.container:visible')).last().parent().find('.zh-floating-control');
						$scope.openSearchAction(floatingControl);
					}, 100);
				}

				$(window).off('click.floatingControl');
				$(window).on('click.floatingControl', function(e) {
					if (!element.find('.search-action').closest('.float-action').hasClass('opened')) {			
						if ($(e.target).parents('.zh-floating-control').length === 0) {
							widget.closeActionMenu();
						}
					}
				});

				$scope.filterFields = function(field) {
					return field.aclIsVisible && !field.skipSearch;
				};

				$scope.toggleSearchFieldsList = function(event) {
					// if filter action exists user cannot choose search field... it will be done in filter popup
					if (!widget.floatingControl.customizationAction || !widget.floatingControl.customizationAction.filter) {
						var el = $(event.target).closest(".floating-card").find('.floating-card-search-field-select');
						el.toggleClass("opened");

						if (el.hasClass("opened")) {
							$(event.target).closest(".floating-card").find('.floating-card-select-field-options').removeClass("opened");
							$(event.target).closest(".floating-card").find('.zh-select-search-floating').removeClass("opened");
						}
					}
				};

				$scope.toggleSelectFieldValueList = function(event) {
					var el = $(event.target).closest(".zh-select-search-floating");
					el.toggleClass("opened");
					var list = $(event.target).closest(".floating-card").find('.floating-card-select-field-options');
					list.toggleClass("opened");

					if (list.hasClass("opened")) {
						$(event.target).closest(".floating-card").find('.floating-card-search-field-select').removeClass("opened");
					}
				};

				$scope.selectSearchField = function(field) {
					$scope.searchField = field;
					element.find('.floating-card-search-field-select').removeClass('opened');
					element.find('input.zh-input-search-floating').focus();
				};

				$scope.selectSearchFieldValue = function(widget, event) {
					var dataValue = $(event.target).attr('data-value');
					var field = $scope.searchField;
					if (dataValue) {
						widget.searchCriteria = widget.searchCriteria || {};
						widget.searchCriteria[field.name] = field.dataSource.data[parseInt(dataValue)][field.valueField].toString();
					} else {
						delete widget.searchCriteria[field.name];
					}

					$(event.target).closest('.floating-card').find('.floating-card-select-field-options').removeClass('opened');
					$(event.target).closest('.floating-card').find('.zh-select-search-floating').removeClass('opened');
				};

				$scope.getSearchFieldValue = function(widget) {
					var field = $scope.searchField;
					var fieldValue;
					var row = field.dataSource.data.filter(function(row) {
						return widget.searchCriteria && widget.searchCriteria[field.name] == row[field.valueField].toString();
					});
					if (row.length > 0) {
						fieldValue = row[0][field.descriptionField];
					}
					return fieldValue;
				};

				if (widget.floatingControl.defaultSearchField) {
					$scope.searchField = widget.getField(widget.floatingControl.defaultSearchField);
				}
			}
		};
	}
})();
