
// FILE: src/services/ZhuLogService.js
function ZhuLogService() {

    this.getPlainContentRows = function(value) {
        var data = [];
        var objValue = JSON.parse(value);

        createRows(data, objValue);
        return data;
    };

    function createRows(data, value, parent) {
        Object.keys(value).forEach(function(key) {
            createRow(data, key, value[key], parent);
        });
    }

    function createRow(data, key, value, parent) {
        var method;
        if (Util.isObject(value)) {
            method = factoryCreateObjectRowsMethod();
        } else if (Array.isArray(value)) {
            method = factoryCreateArrayRowsMethod();
        } else {
            method = createSimpleRow;
        }

        method(data, key, value, parent);
    }

    function factoryCreateObjectRowsMethod() {
        return factoryCreateObjectRows(': {}');
    }

    function factoryCreateArrayRowsMethod() {
        return factoryCreateObjectRows(': []');
    }

    function createSimpleRow(data, key, value, parent) {
        if (typeof value === 'string') {
            value = '"' + value + '"';
        }

        value = key + ': ' + value;
        addData(key, value, parent, data);
    }

    function addData(key, value, parent, data) {
        var id = parent + '#' + key;
        data.push({
            'ID': id,
            'VALUE': value,
            'PARENT': parent
        });

        return id;
    }

    function factoryCreateObjectRows(emptySufix) {
        return function createObjectRows(data, key, rowContent, parent) {
            var rowValue = key;
            if (Object.keys(rowContent).length === 0) {
                rowValue += emptySufix;
            }

            var id = addData(key, rowValue, parent, data);
            createRows(data, rowContent, id);
        };
    }

}

Configuration(function (ContextRegister) {
    ContextRegister.register('ZhuLogService', ZhuLogService);
});

// FILE: src/controllers/ZhuLogController.js
function ZhuLogController(ZhuLogService, ScreenService, templateManager, ZhGridFilter, metaDataFactory) {

    var LOG_WIDGET_NAME = 'zhuLog';
    var CONTENT_WIDGET = 'content';
    var CONTENT_FORM_WIDGET = 'contentForm';
    var CONTENT_GRID_WIDGET = 'contentGrid';
    var CONTENT_FILTER_WIDGET = 'select';

    function getContentWidget() {
        return templateManager.container.getWidget(CONTENT_WIDGET);
    }

    function getContentFormWidget() {
        return templateManager.container.getWidget(CONTENT_FORM_WIDGET);
    }

    function getContentGridWidget() {
        return templateManager.container.getWidget(CONTENT_GRID_WIDGET);
    }

    function getFilterWidget() {
        return templateManager.container.getWidget(CONTENT_FILTER_WIDGET);
    }

    function getZhuLogWidget() {
        return templateManager.container.getWidget(LOG_WIDGET_NAME);
    }

    this.openContent = function(widget) {
        var contentFormWidget = getContentFormWidget();
        contentFormWidget.currentRow = widget.currentRow;

        var rows = ZhuLogService.getPlainContentRows(widget.currentRow.CONTENT);
        getContentGridWidget().dataSource.data = rows;

        ScreenService.openSwipe(getContentWidget());
    };

    var startFilter = function(widget, currentRow) {
        ZhGridFilter.startFiltering(widget);
        var filterPopup = widget.__filterPopup;
        filterPopup.currentRow = currentRow;
    };

    var isAppliedFilter = function(widget){
        return widget.filterApplied && widget.filterApplied.length > 0;
    };

    this.zhuLogWidgetOnEnter = function() {
        var widget = getZhuLogWidget();
        if(!widget.hasFilterApplied){
            if (isAppliedFilter(widget)) {
                widget.reload();
            } else {
                if(widget.__filterPopup ) {
                    startFilter(widget, widget.__filterPopup.currentRow);
                } else if(widget.currentRowDefaultFilter){
                    startFilter(widget, widget.currentRowDefaultFilter);
                } else {
                    startFilter(widget,{});
                }
            }
        }
    };

    this.openFilterPopUp = function() {
        ScreenService.openPopup(getFilterWidget());
    };

    this.setSelectValues = function() {
        var selectWidget = templateManager.container.getWidget('select');
        var select2 = templateManager.container.getWidget('select').fields[1];

        if (selectWidget.currentRow.SNATIVE === 'REQUEST') {
            select2.label = 'Resquest type';
            select2.readOnly = false;

            select2.dataSource.data = [];
            select2.dataSource.data.push(
                {'value': 'Dataset'},
                {'value': 'Row'},
                {'value': 'Empty'},
                {'value': 'Filter'}
            );
        } else {
            if (selectWidget.currentRow.SNATIVE === 'RESPONSE') {
                select2.label = 'Response type';
                select2.readOnly = false;

                select2.dataSource.data = [];
                select2.dataSource.data.push(
                    {
                        'value': 'Dataset'
                    },
                    {
                        'value': 'Message'
                    }
                );
            } else {
                select2.dataSource.data = [];
                this.clearFieldsValue(1);
                select2.readOnly = true;
            }
        }
    };

    this.setTextEdit = function() {
        var selectWidget = templateManager.container.getWidget('select');

        selectWidget.actions[2].isVisible = true;
        selectWidget.actions[3].isVisible = true;
        // selectWidget.fields[2].isVisible = true;
        // selectWidget.fields[3].isVisible = true;

        if (selectWidget.currentRow.SNATIVE === 'REQUEST') {
            if (selectWidget.currentRow.SNATIVE2 === 'Dataset' || selectWidget.currentRow.SNATIVE2 === 'Row') {
                selectWidget.fields[2].label = 'Column name';
                selectWidget.fields[3].label = 'Operator';
                selectWidget.fields[3].template = 'field/select-native.html';
                selectWidget.fields[4].label = 'Value';
                this.addFields();
            }

            if (selectWidget.currentRow.SNATIVE2 === 'Empty') {
                selectWidget.fields[2].label = 'Parameter';
                selectWidget.fields[3].label = 'Operator';
                selectWidget.fields[3].template = 'field/select-native.html';
                selectWidget.fields[4].label = 'Value';

                this.addFields();
            }

            if (selectWidget.currentRow.SNATIVE2 === 'Filter') {
                selectWidget.fields[2].label = 'Column name';
                selectWidget.fields[3].label = 'Column value';
                selectWidget.fields[3].template = 'field/text-edit.html';
                selectWidget.fields[4].label = 'Value';

                this.addFields();
            }

            if (selectWidget.currentRow.SNATIVE2 === null) {
                this.removeExtraFields();
            }
        } else {
            if (selectWidget.currentRow.SNATIVE === 'RESPONSE') {
                if (selectWidget.currentRow.SNATIVE2 === 'Dataset') {
                    selectWidget.fields[2].label = 'Column name';
                    selectWidget.fields[3].label = 'Operator';
                    selectWidget.fields[3].template = 'field/select-native.html';
                    selectWidget.fields[4].label = 'Value';
                    this.addFields();
                }

                if (selectWidget.currentRow.SNATIVE2 == 'Message'){
                    selectWidget.fields[2].label = 'Column name';
                    selectWidget.fields[3].label = 'Operator';
                    selectWidget.fields[3].template = 'field/select-native.html';
                    selectWidget.fields[4].label = 'Value';
                    this.addFields();
                }

                if (selectWidget.currentRow.SNATIVE2 === null) {
                    this.removeExtraFields();
                }
            }
        }
    };

    this.removeExtraFields = function() {
        var selectWidget = templateManager.container.getWidget('select');
        var fields = selectWidget.fields;

        for (var i = (fields.length - 1); i > 6; i--) {
            if (!Util.isUndefined(fields[i])) {
                delete selectWidget.currentRow[fields[i].name];
                delete fields[i];
            }
        }
    };

    this.addFields = function() {
        var selectWidget = templateManager.container.getWidget('select');
        var fields = templateManager.container.getWidget('select').fields;
        var field;
        var dataSource;

        for (var i = 2; i <= 6; i++) {
            if (i !== 5) {
                var data;

                if (!Util.isUndefined(fields[i].dataSource)) {
                    data = fields[i].dataSource.data;
                    dataSource = {
                        'data': data
                    };
                }

                field = {
                    label: fields[i].label,
                    template: fields[i].template,
                    name: 'extraField' + fields.length,
                    class: fields[i].class,
                    events: fields[i].events,
                    dataSource: dataSource,
                    valueField: fields[i].valueField,
                    descriptionField: fields[i].descriptionField,
                    isVisible: true,
                    breakline: fields[i].breakline
                };

                metaDataFactory.fieldFactory(field, selectWidget, selectWidget);
            } else {
                field = {
                    label: fields[i].label,
                    template: fields[i].template,
                    name: 'extraField' + fields.length,
                    class: fields[i].class,
                    events: fields[i].events,
                    dataSource: dataSource,
                    valueField: fields[i].valueField,
                    descriptionField: fields[i].descriptionField,
                    isVisible: false,
                    breakline: fields[i].breakline
                };

                metaDataFactory.fieldFactory(field, selectWidget, selectWidget);
            }
        }
    };

    this.clearAllFields = function() {
        this.removeExtraFields();
        this.clearFieldsValue(0);
    };

    this.clearFieldsValue = function(startingValue) {
        var selectWidget = templateManager.container.getWidget('select');

        for (var i = startingValue; i < selectWidget.fields.length; i++) {
            if(!Util.isUndefined(selectWidget.fields[i])){
                if(selectWidget.fields[i].value !== ''){
                    selectWidget.fields[i].value('');
                }
            }
        }

        selectWidget.fields[1].readOnly = true;
    };

    this.clearFields = function(args) {
        var fieldIndex = args.owner.field.sequenceInList;
        fieldIndex--;

        var selectWidget = templateManager.container.getWidget('select');
        var fields = selectWidget.fields;

        for (var i = fieldIndex; i >= fieldIndex - 4; i--) {

            delete selectWidget.currentRow[fields[i].name];
            delete fields[i];
        }

        templateManager.updateTemplate();
    };

    this.manageFields = function(args) {
        var operatorIndex = args.owner.field.sequenceInList;
        operatorIndex--;

        var selectWidget = templateManager.container.getWidget('select');
        var selectedfield = selectWidget.fields[operatorIndex];
        var selectedValue = selectedfield.value();

        if (selectedValue === 'IS_NULL' || selectedValue === 'IS_NOT_NULL'){
            selectWidget.fields[operatorIndex + 1].readOnly = false;
            selectWidget.fields[operatorIndex + 1].class = 4;
            selectWidget.fields[operatorIndex + 2].isVisible = false;
            selectWidget.fields[operatorIndex + 1].readOnly = true;
        } else if (selectedValue === 'BETWEEN' || selectedValue === 'NOT_BETWEEN') {
            selectWidget.fields[operatorIndex + 1].class = 2;
            selectWidget.fields[operatorIndex + 1].label = 'Begin';
            selectWidget.fields[operatorIndex + 2].isVisible = true;
        } else {
            selectWidget.fields[operatorIndex + 1].readOnly = false;
            selectWidget.fields[operatorIndex + 1].class = 4;
            selectWidget.fields[operatorIndex + 1].label = 'Value';
            selectWidget.fields[operatorIndex + 2].isVisible = false;
        }
    };

    this.startFiltering = function() {
        var selectWidget = templateManager.container.getWidget('select');
        var fields = selectWidget.fields;

        var type = fields[0].value();
        var req_type = fields[1].value();
        var container = fields[2].value();
        var widget = fields[3].value();

        var params = [];
        var lastposition = fields.length - 1;

        var i = 9;
        var j = 0;

        while (i < fields.length) {
            if (!Util.isUndefined(fields[i])) {
                var name = fields[i].value();
                i++;
                var operator = fields[i].value();
                i++;

                var value;
                if(operator === 'IS_NULL' || operator === 'IS_NOT_NULL') {

                    value = '';
                    i += 3;

                } else if(operator === 'BETWEEN' || operator === 'NOT_BETWEEN') {
                    value = [];
                    value[0] = fields[i].value();
                    i++;
                    value[1] = fields[i].value();
                    i+= 2;
                } else {
                    value = fields[i].value();
                    i += 3;
                }

                params[j] = {
                    name: name,
                    operator: operator,
                    value: value
                };
                j++;
            } else {
                i+= 5;
            }
        }

        var resultObject = [
            {
                type: type,
                re_type: req_type,
                container: container,
                widget: widget,
                params: params
            }
        ];

        var result = JSON.stringify(resultObject[0]);
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('ZhuLogController', ZhuLogController);
});

// FILE: src/ZhuLogDecorator.js
function ZhuLogDecorator(ApplicationContext, templateManager) {

    var LOG_CONTAINER_NAME = 'zhulog#log';
    var GRID_WIDGET_INDEX = 0;

    var logDecorator;
    function getLogDecorator() {
        if (logDecorator === undefined) {
            logDecorator = templateManager.projectPromise.then(function() {
                var decoratorServiceName = templateManager.project.zhuLogDecoratorService;
                var logDecorator = ApplicationContext[decoratorServiceName];

                return logDecorator;
            });
        }
        return logDecorator;
    }

    this.afterLoadDataSource = function(args) {
        getLogDecorator().then(function(logDecorator) {
            if (logDecorator) {
                var dataSource = args.owner.dataSource;
                dataSource.data = logDecorator.normalizeRows(dataSource.data);
            }
        });
    };

    this.beforeGridFilterApply = function(args) {
        getLogDecorator().then(function(logDecorator) {
            if (logDecorator) {
                var _event = args.e;
                _event.filters = logDecorator.normalizeFilters(_event.filters);
            }
        });
    };

    this.apply = function(containerName, container) {
        return getLogDecorator().then(function(logDecorator) {
            if (logDecorator && containerName === LOG_CONTAINER_NAME) {
                var logWidget = container.widgets[GRID_WIDGET_INDEX];

                logWidget.events.push({
                    "name": "WidgetAfterLoadDataSource",
                    "code": "ZhuLogDecorator.afterLoadDataSource(args);"
                });

                logWidget.events.push({
                    "name": "GridFilterBeforeApply",
                    "code": "ZhuLogDecorator.beforeGridFilterApply(args);"
                });

                logDecorator.decorateWidget(logWidget);
            }

            return container;
        });
    };

}

Configuration(function (ContainerDecoratorProvider, ContextRegister, ApplicationContext, templateManager) {
    ContextRegister.register('ZhuLogDecorator', ZhuLogDecorator);
    ContainerDecoratorProvider.registerProvider(new ZhuLogDecorator(ApplicationContext, templateManager));
});

// FILE: src/ZhuLogHelper.js
function ZhuLogHelper() {

    this.moveField = function(widget, fieldName, position) {
        var index = -1;
        widget.fields.find(function(field) {
            index++;
            return field.name === fieldName;
        });
        widget.fields.splice(position - 1, 0, widget.fields.splice(index, 1)[0]);
    };

    this.getField = function(widget, fieldName) {
        var field = widget.fields.find(function(field) {
            return field.name === fieldName;
        });

        if (field === null) {
            throw 'Field "'+fieldName+'" not found';
        }

        return field;
    };

    this.addField = function(widget, field, position) {
        position = (position || widget.fields.length) - 1;
        widget.fields.splice(position, 0, field);
    };

    this.removeField = function(widget, fieldName) {
        widget.fields = widget.fields.filter(function(field) {
            return field.name !== fieldName;
        });
    };

    this.editField = function(widget, fieldName, editings) {
        var field = this.getField(widget, fieldName);
        Util.extend(field, editings);
    };

    this.getFilters = function(filters, filterName) {
        return filters.filter(function(filter) {
            return filter.name === filterName;
        });
    };

    this.addFilter = function(filters, filter) {
        filters.push(filter);
    };

    this.removeFilter = function(filters, filterName) {
        filters = filters.filter(function(filter) {
            return filter.name !== filterName;
        });
    };

}

Configuration(function(ContextRegister) {
    ContextRegister.register('ZhuLogHelper', ZhuLogHelper);
});